﻿using System;
using System.Windows.Forms;
using System.IO.Ports;
using System.Threading;
using System.Diagnostics;
using System.IO;


namespace IntelMvbrEspecCommunicationsTest
{
  public partial class CommunicationsTest : Form
  {
    SerialPort serialport = null;
    string strRecvData = "";
    bool bDataReceived = false;
    int nBaudRate = 19200;         // set your baud rate here.  Make sure it matches the communications settings on 8800
    string strCommPort = "COM1";   // set your com port that is used to communicate to 8800 chamber controller
    // set 8800 to no Parity, 8 bits per char, one StopBit.  Terminator:  Last input

    public CommunicationsTest()
    {
      InitializeComponent();
      string strMyDocuments = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
      string strTraceLogPath = strMyDocuments + @"\tracelog\";
      if (!Directory.Exists(strTraceLogPath))
      {
        DirectoryInfo di = Directory.CreateDirectory(strTraceLogPath);
      }
      string strDateTime = string.Format("{0:yyyy-MM-dd_hh-mm-ss-tt}", DateTime.Now);
      strTraceLogPath += strDateTime + ".txt";
      TextWriterTraceListener TwTraceListener = new TextWriterTraceListener(strTraceLogPath);
      Trace.Listeners.Add(TwTraceListener);
      Trace.WriteLine("Serial Communications Test program initiated " + DateTime.Now); Trace.Flush();
      MessageBox.Show("Trace output stored here: " + strTraceLogPath);

      InitializeSerialPort();
    }

    ~CommunicationsTest()
    {
      serialport.Close();
      serialport = null;
      Trace.Flush();
      Trace.Close();
    }

    private void InitializeSerialPort()
    {
      Trace.WriteLine("Initialize Serial Port"); Trace.Flush();
      serialport = new SerialPort(strCommPort, nBaudRate, Parity.None, 8, StopBits.One);
      serialport.Handshake = Handshake.None;
      serialport.ReadTimeout = 300;
      serialport.WriteTimeout = 300;
      try
      {
        if (!(serialport.IsOpen))
        {
          serialport.Open();
        }
      }
      catch (Exception)
      {
        Trace.WriteLine("Could not open serial port COM1"); Trace.Flush();
        MessageBox.Show("Could not open serial port COM1");
        serialport = null;
        return;
      }
      serialport.DiscardInBuffer();

    }

    private string SendReceiveString(string strMsg)
    {
      if (serialport == null)
      {
        return "NaN";
      }

      Trace.WriteLine("SendReceiveString=" + strMsg); Trace.Flush();
      bDataReceived = false; // get ready for next string
      serialport.DiscardInBuffer();
      serialport.Write(strMsg + "\n");

      try
      {
        strRecvData = serialport.ReadLine();
      }
      catch (Exception)
      {
        strRecvData = "NaN";
      }

      Trace.WriteLine("strRecvData=" + strRecvData); Trace.Flush();
      return strRecvData;
    }

    private bool Send(string strMsg)
    {
      if (serialport == null)
      {
        return false;
      }

      Trace.WriteLine("Send=" + strMsg); Trace.Flush();
      serialport.DiscardInBuffer();
      serialport.Write(strMsg + "\n");
      return true;
    }

    public long GetElapsedTimeInMs(long baseTicks)
    {
      long elapsedTicks = DateTime.Now.Ticks - baseTicks;
      return (elapsedTicks /= 10000);  // convert from ticks to mS
    }


    private void AcquireData_button_Click(object sender, EventArgs e)
    {
      Trace.WriteLine("Acquire Data button clicked"); Trace.Flush();
      string strValue = SendReceiveString("VRSN?");
      Version_textBox.Text = strValue;
      Thread.Sleep(1);
      strValue = SendReceiveString("PVAR1?");
      ProcessVariableCh1_textBox.Text = strValue;
      Thread.Sleep(1);
      strValue = SendReceiveString("SETP1?");
      SetpointCh1_textBox.Text = strValue;
      Thread.Sleep(1);
      strValue = SendReceiveString("IDEN?");
      Identification_textBox.Text = strValue;
      Thread.Sleep(1);
      strValue = SendReceiveString("*idn?");
      idn_textBox.Text = strValue;
      Thread.Sleep(1);
      strValue = SendReceiveString("THTL1?");
      ThrottleCh1_textBox.Text = strValue;
      Thread.Sleep(1);
      strValue = SendReceiveString("STAT?");
      Status_textBox.Text = strValue;
      Thread.Sleep(1);
      strValue = SendReceiveString("SERL?");
      SerialNumber_textBox.Text = strValue;
      Thread.Sleep(1);
      strValue = SendReceiveString("SCOD?");
      StopCode_textBox.Text = strValue;
      Thread.Sleep(1);
      strValue = SendReceiveString("RLTM?");
      RealTime_textBox.Text = strValue;
      Thread.Sleep(1);
      strValue = SendReceiveString("OPTN?");
      Options_textBox.Text = strValue;
      Thread.Sleep(1);
    }
  }
}
